﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Persistence;

namespace HeuristicLab.Problems.VehicleRouting.ProblemInstances {
  [StorableType("dde417ca-cb1e-4aa8-b573-22e5451f0d4a")]
  public class StopInsertionInfo {
    private int start;

    public int Start {
      get { return start; }
    }

    private int end;

    public int End {
      get { return end; }
    }

    public StopInsertionInfo(int start, int end)
      : base() {
      this.start = start;
      this.end = end;
    }
  }

  [StorableType("e80a6db1-bd54-46c5-96ed-e2b811c9b054")]
  public class TourInsertionInfo {
    public double Penalty { get; set; }
    public double Quality { get; set; }

    public int Vehicle { get; set; }

    private List<StopInsertionInfo> stopInsertionInfos;

    public TourInsertionInfo(int vehicle)
      : base() {
      stopInsertionInfos = new List<StopInsertionInfo>();
      Vehicle = vehicle;
    }

    public void AddStopInsertionInfo(StopInsertionInfo info) {
      stopInsertionInfos.Add(info);
    }

    public StopInsertionInfo GetStopInsertionInfo(int stop) {
      return stopInsertionInfos[stop];
    }

    public int GetStopCount() {
      return stopInsertionInfos.Count;
    }
  }

  [StorableType("6d3e38c2-65fe-494e-b683-eb77249df833")]
  public class InsertionInfo {
    private List<TourInsertionInfo> tourInsertionInfos;

    public InsertionInfo()
      : base() {
      tourInsertionInfos = new List<TourInsertionInfo>();
    }

    public void AddTourInsertionInfo(TourInsertionInfo info) {
      tourInsertionInfos.Add(info);
    }

    public TourInsertionInfo GetTourInsertionInfo(int tour) {
      return tourInsertionInfos[tour];
    }
  }

  [StorableType("6786408a-87bb-43db-9d44-1aefdf2548e6")]
  public class VRPEvaluation {
    public double Quality { get; set; }
    public double Distance { get; set; }
    public int VehicleUtilization { get; set; }
    public InsertionInfo InsertionInfo { get; set; }

    public double Penalty { get; set; }

    public VRPEvaluation() {
      InsertionInfo = new InsertionInfo();
    }
  }
}
